#ifndef AccountSpielPartner_H
#define AccountSpielPartner_H

#include <Fortschritt.h>
#include <Fenster.h>
#include <Knopf.h>
#include <AuswahlBox.h>
#include <Thread.h>
#include <Rahmen.h>

using namespace Framework;

namespace ASPLEDStyle
{
	const int SpielArt = 1;
	const int Karte = 2;
	const int Spieler = 4;
	const int Ausklappbar = 8;
}

struct AccountSPListeEintragDaten
{
	int style;
	int id;
	Text* name;
	int anzahl;
	int prozent;
};

class AccountSPListeEintrag : public virtual ReferenceCounter
{
private:
	LRahmen* ram;
	RCArray< AccountSPListeEintrag >* members;
	TextFeld* name;
	TextFeld* anzahl;
	FBalken* prozent;
	Knopf* details;
	Bild* ausklappen;
	Bild* einklappen;
	AccountSPListeEintragDaten* daten;
	Text* sortSpalte;
	bool sortAbsteigend;
	double tickVal;
	bool rend;

	// privat
	int getReihenfolge(int* arr);

public:
	// Konstruktor
	AccountSPListeEintrag(AccountSPListeEintragDaten* daten);
	// Destruktor
	~AccountSPListeEintrag();
	// nicht constant
	void addMember(AccountSPListeEintrag* e);
	void setSortSpalte(const char* sp);
	void setSortRichtung(bool absteigend);
	bool tick(double tickVal);
	void doPublicMausEreignis(MausEreignis& me);
	void render(int xOff, int yOff, int breite, Bild& zRObj);
	// constant
	int getHeight() const;
	AccountSPListeEintragDaten* zDaten() const;
};

class AccountSPListe : public Zeichnung
{
private:
	LRahmen* ram;
	VScrollBar* scroll;
	RCArray< AccountSPListeEintrag >* skp;
	RCArray< AccountSPListeEintrag >* spk;
	RCArray< AccountSPListeEintrag >* psk;
	Text* gruppierung;
	Text* sortSpalte;
	bool sortAbsteigend;
	bool rend;

	// privat
	int getReihenfolge(int* arr);

public:
	// Konstruktor
	AccountSPListe();
	// Destruktor
	~AccountSPListe();
	// nicht constant
	void reset();
	void ladeDaten(int accId);
	void setGruppierung(const char* gp);
	void setSortSpalte(const char* sp);
	void setSortRichtung(bool absteigend);
	bool tick(double tickVal) override;
	void doPublicMausEreignis(MausEreignis& me) override;
	void render(Bild& zRObj) override;
};

class AccountSpielPartner : public Thread
{
private:
	Fenster* spielPartnerF;
	AuswahlBox* gruppAusw;
	AuswahlBox* sortSpalte;
	AuswahlBox* sortRichtung;
	AccountSPListe* liste;
	int status;
	int accId;
	int animation;
	unsigned char alpha;
	double tickVal;
	bool rend;

public:
	// Konstruktor
	AccountSpielPartner();
	// Destruktor
	~AccountSpielPartner();
	// nicht constant
	void reset();
	void ladeStatistik(int accId);
	virtual void thread();
	void setSichtbar(bool sichtbar, bool nachRechts);
	bool tick(double zeit);
	void doPublicMausEreignis(MausEreignis& me);
	void render(Bild& zRObj);
	// constant
	int getStatus() const;
};

#endif;