#ifndef Gruppe_H
#define Gruppe_H

#include <Klient.h>
#include <Thread.h>
#include <Array.h>
#include <Knopf.h>
#include <Scroll.h>
#include <Fenster.h>
#include <KSGScript.h>
#include <Rahmen.h>

using namespace Framework;
using namespace KSGScript;

class GruppeEinladungAccount : public virtual ReferenceCounter
{
private:
    LRahmen *rahmen;
    AlphaFeld *auswahl;
    TextFeld *name;
    Knopf *nachrichtSenden;
    Knopf *erneutSenden;
    Knopf *einladungAbbrechen;
    int status;
    int br;
    int knopfX;
    double tickVal;
    bool mausIn;
    bool remove;
    int accountId;
    int gruppeId;
    int admin;
    int hhe;
    int animation;
    bool rend;

public:
    // Konstruktor
    GruppeEinladungAccount();
    // Destruktor
    ~GruppeEinladungAccount();
    // nicht constant
    void setGruppeId( int gruppeId );
    void setAccountId( int id );
    void setSize( int br );
    void setAbgelehnt();
    void setAngenommen();
    void setRemove();
    void setAdmin( int admin );
    bool tick( double tickVal );
    void doPublicMausEreignis( MausEreignis &me );
    void render( int yOff, Bild &zrObj );
    // constant
    bool getRemove() const;
    int getAccountId() const;
    int getHeight() const;
};

class GruppeEinladungListe : public virtual ReferenceCounter
{
private:
    Punkt pos;
    Punkt gr;
    LRahmen *rahmen;
    TextFeld *einladenName;
    Knopf *einladen;
    VScrollBar *scroll;
    RCArray< GruppeEinladungAccount > *accounts;
    int einladungHhe;
    double tickVal;
    int anzahl;
    int gruppeId;
    int admin;
    bool rend;

public:
    // Konstruktor
    GruppeEinladungListe();
    // Destruktor
    ~GruppeEinladungListe();
    // nicht constant
    void reset();
    void setGruppeId( int gruppeId );
    void addAccount( int id );
    void setAbgelehnt( int id );
    void setAngenommen( int id );
    void remove( int id );
    void setAdmin( int admin );
    bool tick( double tickVal );
    void doPublicMausEreignis( MausEreignis &me );
    void doTastaturEreignis( TastaturEreignis &te );
    void render( Bild &zRObj );
};

class GruppeAccountDaten : public virtual ReferenceCounter
{
private:
    LRahmen *rahmen;
    AlphaFeld *auswahl;
    TextFeld *name;
    TextFeld *punkte;
    Knopf *nachrichtSenden;
    Knopf *accountAnsehen;
    Knopf *freundEinladung;
    Knopf *kick;
    int br;
    double tickVal;
    bool mausIn;
    bool remove;
    int gruppeId;
    int accountId;
    int admin;
    int hhe;
    int animation;
    bool rend;

public:
    // Konstruktor
    GruppeAccountDaten();
    // Destruktor
    ~GruppeAccountDaten();
    // nicht constant
    void setGruppeId( int gruppeId );
    void setAccountId( int id );
    void setAdmin( int admin );
    void setSize( int x );
    void entfernen();
    bool tick( double tickVal );
    void doPublicMausEreignis( MausEreignis &me );
    void render( int y, Bild &zrObj );
    // constant
    bool getRemove() const;
    int getAccountId() const;
    int getHeight() const;
};

class GruppeAccountListe : public virtual ReferenceCounter
{
private:
    Punkt pos;
    Punkt gr;
    LRahmen *rahmen;
    VScrollBar *scroll;
    RCArray< GruppeAccountDaten > *accounts;
    int anzahl;
    int gruppeId;
    int admin;
    bool rend;

public:
    // Konstruktor
    GruppeAccountListe();
    // Destruktor
    ~GruppeAccountListe();
    // nicht constant
    void reset();
    void setGruppeId( int gruppeId );
    void addAccount( int id );
    void removeAccount( int id );
    void setAdmin( int admin );
    bool tick( double tickVal );
    void doPublicMausEreignis( MausEreignis &me );
    void render( Bild &zRObj );
};

class GruppeChat : public virtual ReferenceCounter
{
private:
    Punkt pos;
    Punkt gr;
    LRahmen *rahmen;
    TextFeld *nachricht;
    Knopf *senden;
    TextFeld *verlauf;
    int gruppeId;
    bool rend;

public:
    // Konstruktor
    GruppeChat();
    // Destruktor
    ~GruppeChat();
    // nicht constant
    void reset();
    void setGruppeId( int gruppeId );
    void gruppeNachricht( const char *nachricht );
    bool tick( double tickVal );
    void doPublicMausEreignis( MausEreignis &me );
    void doTastaturEreignis( TastaturEreignis &te );
    void render( Bild &zRObj );
};

class GruppeAnmeldung : public virtual ReferenceCounter
{
private:
    Punkt pos;
    Punkt gr;
    LRahmen *rahmen;
    KontrollKnopf *spielerHinzufgen;
    KontrollKnopf *angemeldet;
    TextFeld *zeit;
    Knopf *gruppeVerlassen;
    int admin;
    double time;
    int gruppeId;
    bool rend;

public:
    // Konstruktor
    GruppeAnmeldung();
    // Destruktor
    ~GruppeAnmeldung();
    // nicht constant
    void reset();
    void setGruppeId( int gruppeId );
    void setSpielerHinzufgen( bool spielerHinzufgen );
    void setAngemeldet( bool angemeldet );
    void zurckInWarteschlange( int stunden, int minuten, int sekunden );
    void spielGefunden();
    void setAdmin( int admin );
    bool tick( double tickVal );
    void doPublicMausEreignis( MausEreignis &me );
    void render( Bild &zRObj );
};

class GruppeKarte : public Thread
{
private:
    Punkt pos;
    Punkt gr;
    LRahmen *rahmen;
    Bild *hintergrund;
    KSGScriptObj *info;
    HINSTANCE ksgs;
    TextFeld *fehler;
    TextRenderer *tr;
    unsigned char alpha1;
    unsigned char alpha2;
    double tickVal;
    double ladenZeit;
    int ladeBild;
    int karteId;
    int gruppeId;
    bool exit;
    bool geladen;
    bool rend;

public:
    // Konstruktor
    GruppeKarte();
    // Destruktor
    ~GruppeKarte();
    // nicht constant
    void reset();
    void ksgsAktion( RCArray< KSGSVariable > *parameter, KSGSVariable **retVal );
    void setGruppeId( int gruppeId );
    bool tick( double tickVal );
    void doPublicMausEreignis( MausEreignis &me );
    void render( Bild &zrObj );
    virtual void thread();
    // constant
    int getKarteId() const;
};

class GruppeSpielGefunden : public virtual ReferenceCounter
{
private:
    Punkt pos;
    Punkt gr;
    Fenster *spielGefunden;
    TextFeld *zeit;
    TextFeld *warte;
    Knopf *annehmen;
    unsigned char alpha;
    double time;
    double tickVal;
    bool sichtbar;
    bool rend;

public:
    // Konstruktor
    GruppeSpielGefunden();
    // Destruktor
    ~GruppeSpielGefunden();
    // nicht constant
    void reset();
    void setSichtbar( bool sichtbar );
    void setVerbleibendeZeit( int sekunden );
    bool tick( double tickVal );
    void doPublicMausEreignis( MausEreignis &me );
    void render( Bild &zRObj );
    // constant
    bool istSichtbar() const;
};

class GruppeFenster : public virtual ReferenceCounter
{
private:
    Punkt pos;
    Punkt gr;
    GruppeAnmeldung *anmeldung;
    GruppeChat *chat;
    GruppeAccountListe *accounts;
    GruppeEinladungListe *einladungen;
    GruppeKarte *karte;
    GruppeSpielGefunden *spiel;
    bool sichtbar;
    unsigned char alpha;
    int gruppeId;
    double tickVal;
    bool rend;

public:
    // Konstruktor
    GruppeFenster();
    // Destruktor
    ~GruppeFenster();
    // nicht constant
    void setGruppeId( int id );
    void setSichtbar( bool sichtbar );
    void chatNachricht( const char *nachricht );
    void neuerSpieler( int accountId );
    void spielerVerlsst( int accountId );
    void neueEinladung( int accountId );
    void einladungEntfernt( int accountId );
    void einladungAbgelehnt( int accountId );
    void setSpielerHinzufgen( bool sh );
    void setAngemeldet( bool angemeldet );
    void spielGefunden();
    void verbleibendeZeit( int sekunden );
    void spielGefundenAbbruch();
    void zurckInWarteschlange( int stunden, int minuten, int sekunden );
    void setAdmin( int accountId );
    void kick();
    void reset();
    bool tick( double tickVal );
    void doPublicMausEreignis( MausEreignis &me );
    void doTastaturEreignis( TastaturEreignis &te );
    void render( Bild &zRObj );
    // constant
    int getGruppeId() const;
    int getKarteId() const;
};

#endif